@@ -1,22 +1,49 @@
+// @ts-check
 'use strict'
 
 const tape = require('tape')
 const crypto = require('crypto')
 const fs = require('fs')
 const path = require('path')
-const BufferList = require('../')
-const Buffer = require('safe-buffer').Buffer
+const os = require('os')
+const BufferListStream = require('../')
+const { Buffer } = require('buffer')
 
-const encodings =
-      ('hex utf8 utf-8 ascii binary base64' +
-          (process.browser ? '' : ' ucs2 ucs-2 utf16le utf-16le')).split(' ')
+/**
+ * This typedef allows us to add _bufs to the API without declaring it publicly on types.
+ * @typedef { BufferListStream & { _bufs?: Buffer[] }} BufferListStreamWithPrivate
+ */
+
+/**
+ * Just for typechecking in js
+ * @type { NodeJS.Process & { browser?: boolean }}
+ */
+
+const process = globalThis.process
+
+/** @type {BufferEncoding[]} */
+const encodings = ['ascii', 'utf8', 'utf-8', 'hex', 'binary', 'base64']
+
+if (process.browser) {
+  encodings.push(
+    'ucs2',
+    'ucs-2',
+    'utf16le',
+    /**
+     * This alias is not in typescript typings for BufferEncoding. Still have to fix
+     * @see https://nodejs.org/api/buffer.html#buffers-and-character-encodings
+     */
+    // @ts-ignore
+    'utf-16le'
+  )
+}
 
 require('./indexOf')
 require('./isBufferList')
 require('./convert')
 
 tape('single bytes from single buffer', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('abcd'))
 
@@ -32,7 +59,7 @@ tape('single bytes from single buffer', function (t) {
 })
 
 tape('single bytes from multiple buffers', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('abcd'))
   bl.append(Buffer.from('efg'))
@@ -56,7 +83,7 @@ tape('single bytes from multiple buffers', function (t) {
 })
 
 tape('multi bytes from single buffer', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('abcd'))
 
@@ -71,7 +98,7 @@ tape('multi bytes from single buffer', function (t) {
 })
 
 tape('multi bytes from single buffer (negative indexes)', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('buffer'))
 
@@ -85,7 +112,7 @@ tape('multi bytes from single buffer (negative indexes)', function (t) {
 })
 
 tape('multiple bytes from multiple buffers', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('abcd'))
   bl.append(Buffer.from('efg'))
@@ -105,10 +132,10 @@ tape('multiple bytes from multiple buffers', function (t) {
 })
 
 tape('multiple bytes from multiple buffer lists', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
-  bl.append(new BufferList([Buffer.from('abcd'), Buffer.from('efg')]))
-  bl.append(new BufferList([Buffer.from('hi'), Buffer.from('j')]))
+  bl.append(new BufferListStream([Buffer.from('abcd'), Buffer.from('efg')]))
+  bl.append(new BufferListStream([Buffer.from('hi'), Buffer.from('j')]))
 
   t.equal(bl.length, 10)
 
@@ -124,17 +151,19 @@ tape('multiple bytes from multiple buffer lists', function (t) {
 
 // same data as previous test, just using nested constructors
 tape('multiple bytes from crazy nested buffer lists', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
-  bl.append(new BufferList([
-    new BufferList([
-      new BufferList(Buffer.from('abc')),
-      Buffer.from('d'),
-      new BufferList(Buffer.from('efg'))
-    ]),
-    new BufferList([Buffer.from('hi')]),
-    new BufferList(Buffer.from('j'))
-  ]))
+  bl.append(
+    new BufferListStream([
+      new BufferListStream([
+        new BufferListStream(Buffer.from('abc')),
+        Buffer.from('d'),
+        new BufferListStream(Buffer.from('efg'))
+      ]),
+      new BufferListStream([Buffer.from('hi')]),
+      new BufferListStream(Buffer.from('j'))
+    ])
+  )
 
   t.equal(bl.length, 10)
 
@@ -149,7 +178,7 @@ tape('multiple bytes from crazy nested buffer lists', function (t) {
 })
 
 tape('append accepts arrays of Buffers', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('abc'))
   bl.append([Buffer.from('def')])
@@ -162,7 +191,7 @@ tape('append accepts arrays of Buffers', function (t) {
 })
 
 tape('append accepts arrays of Uint8Arrays', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(new Uint8Array([97, 98, 99]))
   bl.append([Uint8Array.from([100, 101, 102])])
@@ -175,12 +204,17 @@ tape('append accepts arrays of Uint8Arrays', function (t) {
 })
 
 tape('append accepts arrays of BufferLists', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('abc'))
-  bl.append([new BufferList('def')])
-  bl.append(new BufferList([Buffer.from('ghi'), new BufferList('jkl')]))
-  bl.append([Buffer.from('mnop'), new BufferList([Buffer.from('qrstu'), Buffer.from('vwxyz')])])
+  bl.append([new BufferListStream('def')])
+  bl.append(
+    new BufferListStream([Buffer.from('ghi'), new BufferListStream('jkl')])
+  )
+  bl.append([
+    Buffer.from('mnop'),
+    new BufferListStream([Buffer.from('qrstu'), Buffer.from('vwxyz')])
+  ])
   t.equal(bl.length, 26)
   t.equal(bl.slice().toString('ascii'), 'abcdefghijklmnopqrstuvwxyz')
 
@@ -188,21 +222,26 @@ tape('append accepts arrays of BufferLists', function (t) {
 })
 
 tape('append chainable', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   t.ok(bl.append(Buffer.from('abcd')) === bl)
   t.ok(bl.append([Buffer.from('abcd')]) === bl)
-  t.ok(bl.append(new BufferList(Buffer.from('abcd'))) === bl)
-  t.ok(bl.append([new BufferList(Buffer.from('abcd'))]) === bl)
+  t.ok(bl.append(new BufferListStream(Buffer.from('abcd'))) === bl)
+  t.ok(bl.append([new BufferListStream(Buffer.from('abcd'))]) === bl)
 
   t.end()
 })
 
 tape('append chainable (test results)', function (t) {
-  const bl = new BufferList('abc')
-    .append([new BufferList('def')])
-    .append(new BufferList([Buffer.from('ghi'), new BufferList('jkl')]))
-    .append([Buffer.from('mnop'), new BufferList([Buffer.from('qrstu'), Buffer.from('vwxyz')])])
+  const bl = new BufferListStream('abc')
+    .append([new BufferListStream('def')])
+    .append(
+      new BufferListStream([Buffer.from('ghi'), new BufferListStream('jkl')])
+    )
+    .append([
+      Buffer.from('mnop'),
+      new BufferListStream([Buffer.from('qrstu'), Buffer.from('vwxyz')])
+    ])
 
   t.equal(bl.length, 26)
   t.equal(bl.slice().toString('ascii'), 'abcdefghijklmnopqrstuvwxyz')
@@ -211,7 +250,7 @@ tape('append chainable (test results)', function (t) {
 })
 
 tape('consuming from multiple buffers', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('abcd'))
   bl.append(Buffer.from('efg'))
@@ -246,7 +285,8 @@ tape('consuming from multiple buffers', function (t) {
 })
 
 tape('complete consumption', function (t) {
-  const bl = new BufferList()
+  /** @type {BufferListStreamWithPrivate} */
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('a'))
   bl.append(Buffer.from('b'))
@@ -263,8 +303,9 @@ tape('test readUInt8 / readInt8', function (t) {
   const buf1 = Buffer.alloc(1)
   const buf2 = Buffer.alloc(3)
   const buf3 = Buffer.alloc(3)
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
+  buf1[0] = 0x1
   buf2[1] = 0x3
   buf2[2] = 0x4
   buf3[0] = 0x23
@@ -274,6 +315,7 @@ tape('test readUInt8 / readInt8', function (t) {
   bl.append(buf2)
   bl.append(buf3)
 
+  t.equal(bl.readUInt8(), 0x1)
   t.equal(bl.readUInt8(2), 0x3)
   t.equal(bl.readInt8(2), 0x3)
   t.equal(bl.readUInt8(3), 0x4)
@@ -290,8 +332,9 @@ tape('test readUInt16LE / readUInt16BE / readInt16LE / readInt16BE', function (t
   const buf1 = Buffer.alloc(1)
   const buf2 = Buffer.alloc(3)
   const buf3 = Buffer.alloc(3)
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
+  buf1[0] = 0x1
   buf2[1] = 0x3
   buf2[2] = 0x4
   buf3[0] = 0x23
@@ -301,6 +344,8 @@ tape('test readUInt16LE / readUInt16BE / readInt16LE / readInt16BE', function (t
   bl.append(buf2)
   bl.append(buf3)
 
+  t.equal(bl.readUInt16BE(), 0x0100)
+  t.equal(bl.readUInt16LE(), 0x0001)
   t.equal(bl.readUInt16BE(2), 0x0304)
   t.equal(bl.readUInt16LE(2), 0x0403)
   t.equal(bl.readInt16BE(2), 0x0304)
@@ -321,8 +366,9 @@ tape('test readUInt32LE / readUInt32BE / readInt32LE / readInt32BE', function (t
   const buf1 = Buffer.alloc(1)
   const buf2 = Buffer.alloc(3)
   const buf3 = Buffer.alloc(3)
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
+  buf1[0] = 0x1
   buf2[1] = 0x3
   buf2[2] = 0x4
   buf3[0] = 0x23
@@ -332,6 +378,8 @@ tape('test readUInt32LE / readUInt32BE / readInt32LE / readInt32BE', function (t
   bl.append(buf2)
   bl.append(buf3)
 
+  t.equal(bl.readUInt32BE(), 0x01000304)
+  t.equal(bl.readUInt32LE(), 0x04030001)
   t.equal(bl.readUInt32BE(2), 0x03042342)
   t.equal(bl.readUInt32LE(2), 0x42230403)
   t.equal(bl.readInt32BE(2), 0x03042342)
@@ -344,7 +392,7 @@ tape('test readUIntLE / readUIntBE / readIntLE / readIntBE', function (t) {
   const buf1 = Buffer.alloc(1)
   const buf2 = Buffer.alloc(3)
   const buf3 = Buffer.alloc(3)
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   buf2[0] = 0x2
   buf2[1] = 0x3
@@ -389,8 +437,9 @@ tape('test readFloatLE / readFloatBE', function (t) {
   const buf1 = Buffer.alloc(1)
   const buf2 = Buffer.alloc(3)
   const buf3 = Buffer.alloc(3)
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
+  buf1[0] = 0x01
   buf2[1] = 0x00
   buf2[2] = 0x00
   buf3[0] = 0x80
@@ -400,7 +449,11 @@ tape('test readFloatLE / readFloatBE', function (t) {
   bl.append(buf2)
   bl.append(buf3)
 
-  t.equal(bl.readFloatLE(2), 0x01)
+  const canonical = Buffer.concat([buf1, buf2, buf3])
+  t.equal(bl.readFloatLE(), canonical.readFloatLE())
+  t.equal(bl.readFloatBE(), canonical.readFloatBE())
+  t.equal(bl.readFloatLE(2), canonical.readFloatLE(2))
+  t.equal(bl.readFloatBE(2), canonical.readFloatBE(2))
 
   t.end()
 })
@@ -409,8 +462,9 @@ tape('test readDoubleLE / readDoubleBE', function (t) {
   const buf1 = Buffer.alloc(1)
   const buf2 = Buffer.alloc(3)
   const buf3 = Buffer.alloc(10)
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
+  buf1[0] = 0x01
   buf2[1] = 0x55
   buf2[2] = 0x55
   buf3[0] = 0x55
@@ -424,13 +478,17 @@ tape('test readDoubleLE / readDoubleBE', function (t) {
   bl.append(buf2)
   bl.append(buf3)
 
-  t.equal(bl.readDoubleLE(2), 0.3333333333333333)
+  const canonical = Buffer.concat([buf1, buf2, buf3])
+  t.equal(bl.readDoubleBE(), canonical.readDoubleBE())
+  t.equal(bl.readDoubleLE(), canonical.readDoubleLE())
+  t.equal(bl.readDoubleBE(2), canonical.readDoubleBE(2))
+  t.equal(bl.readDoubleLE(2), canonical.readDoubleLE(2))
 
   t.end()
 })
 
 tape('test toString', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
 
   bl.append(Buffer.from('abcd'))
   bl.append(Buffer.from('efg'))
@@ -447,7 +505,7 @@ tape('test toString', function (t) {
 })
 
 tape('test toString encoding', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
   const b = Buffer.from('abcdefghij\xff\x00')
 
   bl.append(Buffer.from('abcd'))
@@ -463,20 +521,36 @@ tape('test toString encoding', function (t) {
   t.end()
 })
 
+tape('uninitialized memory', function (t) {
+  const secret = crypto.randomBytes(256)
+  for (let i = 0; i < 1e6; i++) {
+    const clone = Buffer.from(secret)
+    const bl = new BufferListStream()
+    bl.append(Buffer.from('a'))
+    bl.consume(-1024)
+    const buf = bl.slice(1)
+    if (buf.indexOf(clone) !== -1) {
+      t.fail(`Match (at ${i})`)
+      break
+    }
+  }
+  t.end()
+})
+
 !process.browser && tape('test stream', function (t) {
   const random = crypto.randomBytes(65534)
 
-  const bl = new BufferList((err, buf) => {
+  const bl = new BufferListStream((err, buf) => {
     t.ok(Buffer.isBuffer(buf))
     t.ok(err === null)
     t.ok(random.equals(bl.slice()))
     t.ok(random.equals(buf.slice()))
 
-    bl.pipe(fs.createWriteStream('/tmp/bl_test_rnd_out.dat'))
+    bl.pipe(fs.createWriteStream(path.join(os.tmpdir(), 'bl_test_rnd_out.dat')))
       .on('close', function () {
         const rndhash = crypto.createHash('md5').update(random).digest('hex')
         const md5sum = crypto.createHash('md5')
-        const s = fs.createReadStream('/tmp/bl_test_rnd_out.dat')
+        const s = fs.createReadStream(path.join(os.tmpdir(), 'bl_test_rnd_out.dat'))
 
         s.on('data', md5sum.update.bind(md5sum))
         s.on('end', function () {
@@ -486,24 +560,24 @@ tape('test toString encoding', function (t) {
       })
   })
 
-  fs.writeFileSync('/tmp/bl_test_rnd.dat', random)
-  fs.createReadStream('/tmp/bl_test_rnd.dat').pipe(bl)
+  fs.writeFileSync(path.join(os.tmpdir(), 'bl_test_rnd.dat'), random)
+  fs.createReadStream(path.join(os.tmpdir(), 'bl_test_rnd.dat')).pipe(bl)
 })
 
 tape('instantiation with Buffer', function (t) {
   const buf = crypto.randomBytes(1024)
   const buf2 = crypto.randomBytes(1024)
-  let b = BufferList(buf)
+  let b = BufferListStream(buf)
 
   t.equal(buf.toString('hex'), b.slice().toString('hex'), 'same buffer')
-  b = BufferList([buf, buf2])
+  b = BufferListStream([buf, buf2])
   t.equal(b.slice().toString('hex'), Buffer.concat([buf, buf2]).toString('hex'), 'same buffer')
 
   t.end()
 })
 
 tape('test String appendage', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
   const b = Buffer.from('abcdefghij\xff\x00')
 
   bl.append('abcd')
@@ -520,7 +594,7 @@ tape('test String appendage', function (t) {
 })
 
 tape('test Number appendage', function (t) {
-  const bl = new BufferList()
+  const bl = new BufferListStream()
   const b = Buffer.from('1234567890')
 
   bl.append(1234)
@@ -537,7 +611,7 @@ tape('test Number appendage', function (t) {
 
 tape('write nothing, should get empty buffer', function (t) {
   t.plan(3)
-  BufferList(function (err, data) {
+  BufferListStream(function (err, data) {
     t.notOk(err, 'no error')
     t.ok(Buffer.isBuffer(data), 'got a buffer')
     t.equal(0, data.length, 'got a zero-length buffer')
@@ -551,7 +625,7 @@ tape('unicode string', function (t) {
   const inp1 = '\u2600'
   const inp2 = '\u2603'
   const exp = inp1 + ' and ' + inp2
-  const bl = BufferList()
+  const bl = BufferListStream()
 
   bl.write(inp1)
   bl.write(' and ')
@@ -561,8 +635,8 @@ tape('unicode string', function (t) {
 })
 
 tape('should emit finish', function (t) {
-  const source = BufferList()
-  const dest = BufferList()
+  const source = BufferListStream()
+  const dest = BufferListStream()
 
   source.write('hello')
   source.pipe(dest)
@@ -576,7 +650,7 @@ tape('should emit finish', function (t) {
 tape('basic copy', function (t) {
   const buf = crypto.randomBytes(1024)
   const buf2 = Buffer.alloc(1024)
-  const b = BufferList(buf)
+  const b = BufferListStream(buf)
 
   b.copy(buf2)
   t.equal(b.slice().toString('hex'), buf2.toString('hex'), 'same buffer')
@@ -587,7 +661,7 @@ tape('basic copy', function (t) {
 tape('copy after many appends', function (t) {
   const buf = crypto.randomBytes(512)
   const buf2 = Buffer.alloc(1024)
-  const b = BufferList(buf)
+  const b = BufferListStream(buf)
 
   b.append(buf)
   b.copy(buf2)
@@ -599,7 +673,7 @@ tape('copy after many appends', function (t) {
 tape('copy at a precise position', function (t) {
   const buf = crypto.randomBytes(1004)
   const buf2 = Buffer.alloc(1024)
-  const b = BufferList(buf)
+  const b = BufferListStream(buf)
 
   b.copy(buf2, 20)
   t.equal(b.slice().toString('hex'), buf2.slice(20).toString('hex'), 'same buffer')
@@ -610,7 +684,7 @@ tape('copy at a precise position', function (t) {
 tape('copy starting from a precise location', function (t) {
   const buf = crypto.randomBytes(10)
   const buf2 = Buffer.alloc(5)
-  const b = BufferList(buf)
+  const b = BufferListStream(buf)
 
   b.copy(buf2, 0, 5)
   t.equal(b.slice(5).toString('hex'), buf2.toString('hex'), 'same buffer')
@@ -620,7 +694,7 @@ tape('copy starting from a precise location', function (t) {
 
 tape('copy in an interval', function (t) {
   const rnd = crypto.randomBytes(10)
-  const b = BufferList(rnd) // put the random bytes there
+  const b = BufferListStream(rnd) // put the random bytes there
   const actual = Buffer.alloc(3)
   const expected = Buffer.alloc(3)
 
@@ -635,7 +709,7 @@ tape('copy in an interval', function (t) {
 tape('copy an interval between two buffers', function (t) {
   const buf = crypto.randomBytes(10)
   const buf2 = Buffer.alloc(10)
-  const b = BufferList(buf)
+  const b = BufferListStream(buf)
 
   b.append(buf)
   b.copy(buf2, 0, 5, 15)
@@ -646,7 +720,7 @@ tape('copy an interval between two buffers', function (t) {
 })
 
 tape('shallow slice across buffer boundaries', function (t) {
-  const bl = new BufferList(['First', 'Second', 'Third'])
+  const bl = new BufferListStream(['First', 'Second', 'Third'])
 
   t.equal(bl.shallowSlice(3, 13).toString(), 'stSecondTh')
 
@@ -656,7 +730,7 @@ tape('shallow slice across buffer boundaries', function (t) {
 tape('shallow slice within single buffer', function (t) {
   t.plan(2)
 
-  const bl = new BufferList(['First', 'Second', 'Third'])
+  const bl = new BufferListStream(['First', 'Second', 'Third'])
 
   t.equal(bl.shallowSlice(5, 10).toString(), 'Secon')
   t.equal(bl.shallowSlice(7, 10).toString(), 'con')
@@ -667,7 +741,7 @@ tape('shallow slice within single buffer', function (t) {
 tape('shallow slice single buffer', function (t) {
   t.plan(3)
 
-  const bl = new BufferList(['First', 'Second', 'Third'])
+  const bl = new BufferListStream(['First', 'Second', 'Third'])
 
   t.equal(bl.shallowSlice(0, 5).toString(), 'First')
   t.equal(bl.shallowSlice(5, 11).toString(), 'Second')
@@ -677,7 +751,7 @@ tape('shallow slice single buffer', function (t) {
 tape('shallow slice with negative or omitted indices', function (t) {
   t.plan(4)
 
-  const bl = new BufferList(['First', 'Second', 'Third'])
+  const bl = new BufferListStream(['First', 'Second', 'Third'])
 
   t.equal(bl.shallowSlice().toString(), 'FirstSecondThird')
   t.equal(bl.shallowSlice(5).toString(), 'SecondThird')
@@ -689,7 +763,7 @@ tape('shallow slice does not make a copy', function (t) {
   t.plan(1)
 
   const buffers = [Buffer.from('First'), Buffer.from('Second'), Buffer.from('Third')]
-  const bl = (new BufferList(buffers)).shallowSlice(5, -3)
+  const bl = new BufferListStream(buffers).shallowSlice(5, -3)
 
   buffers[1].fill('h')
   buffers[2].fill('h')
@@ -701,7 +775,7 @@ tape('shallow slice with 0 length', function (t) {
   t.plan(1)
 
   const buffers = [Buffer.from('First'), Buffer.from('Second'), Buffer.from('Third')]
-  const bl = (new BufferList(buffers)).shallowSlice(0, 0)
+  const bl = (new BufferListStream(buffers)).shallowSlice(0, 0)
 
   t.equal(bl.length, 0)
 })
@@ -710,7 +784,7 @@ tape('shallow slice with 0 length from middle', function (t) {
   t.plan(1)
 
   const buffers = [Buffer.from('First'), Buffer.from('Second'), Buffer.from('Third')]
-  const bl = (new BufferList(buffers)).shallowSlice(10, 10)
+  const bl = (new BufferListStream(buffers)).shallowSlice(10, 10)
 
   t.equal(bl.length, 0)
 })
@@ -718,7 +792,7 @@ tape('shallow slice with 0 length from middle', function (t) {
 tape('duplicate', function (t) {
   t.plan(2)
 
-  const bl = new BufferList('abcdefghij\xff\x00')
+  const bl = new BufferListStream('abcdefghij\xff\x00')
   const dup = bl.duplicate()
 
   t.equal(bl.prototype, dup.prototype)
@@ -728,7 +802,8 @@ tape('duplicate', function (t) {
 tape('destroy no pipe', function (t) {
   t.plan(2)
 
-  const bl = new BufferList('alsdkfja;lsdkfja;lsdk')
+  /** @type {BufferListStreamWithPrivate} */
+  const bl = new BufferListStream('alsdkfja;lsdkfja;lsdk')
 
   bl.destroy()
 
@@ -739,7 +814,8 @@ tape('destroy no pipe', function (t) {
 tape('destroy with error', function (t) {
   t.plan(3)
 
-  const bl = new BufferList('alsdkfja;lsdkfja;lsdk')
+  /** @type {BufferListStreamWithPrivate} */
+  const bl = new BufferListStream('alsdkfja;lsdkfja;lsdk')
   const err = new Error('kaboom')
 
   bl.destroy(err)
@@ -754,7 +830,8 @@ tape('destroy with error', function (t) {
 !process.browser && tape('destroy with pipe before read end', function (t) {
   t.plan(2)
 
-  const bl = new BufferList()
+  /** @type {BufferListStreamWithPrivate} */
+  const bl = new BufferListStream()
   fs.createReadStream(path.join(__dirname, '/test.js'))
     .pipe(bl)
 
@@ -767,7 +844,8 @@ tape('destroy with error', function (t) {
 !process.browser && tape('destroy with pipe before read end with race', function (t) {
   t.plan(2)
 
-  const bl = new BufferList()
+  /** @type {BufferListStreamWithPrivate} */
+  const bl = new BufferListStream()
 
   fs.createReadStream(path.join(__dirname, '/test.js'))
     .pipe(bl)
@@ -784,8 +862,8 @@ tape('destroy with error', function (t) {
 !process.browser && tape('destroy with pipe after read end', function (t) {
   t.plan(2)
 
-  const bl = new BufferList()
-
+  /** @type {BufferListStreamWithPrivate} */
+  const bl = new BufferListStream()
   fs.createReadStream(path.join(__dirname, '/test.js'))
     .on('end', onEnd)
     .pipe(bl)
@@ -801,8 +879,9 @@ tape('destroy with error', function (t) {
 !process.browser && tape('destroy with pipe while writing to a destination', function (t) {
   t.plan(4)
 
-  const bl = new BufferList()
-  const ds = new BufferList()
+  /** @type {BufferListStreamWithPrivate} */
+  const bl = new BufferListStream()
+  const ds = new BufferListStream()
 
   fs.createReadStream(path.join(__dirname, '/test.js'))
     .on('end', onEnd)
@@ -828,7 +907,7 @@ tape('destroy with error', function (t) {
 !process.browser && tape('handle error', function (t) {
   t.plan(2)
 
-  fs.createReadStream('/does/not/exist').pipe(BufferList(function (err, data) {
+  fs.createReadStream('/does/not/exist').pipe(BufferListStream(function (err, data) {
     t.ok(err instanceof Error, 'has error')
     t.notOk(data, 'no data')
   }))
